@@ -1,19 +1,33 @@
-# $Id: emoncms.py 892 2014-03-13 04:16:32Z mwall $
+# $Id: emoncms.py 971 2014-07-28 23:16:44Z mwall $
 # Copyright 2013 Matthew Wall
 
-#==============================================================================
-# EmonCMS
-#==============================================================================
 # Upload data to EmonCMS
 # http://emoncms.org
 #
 # Installation:
 # 1) put this file in bin/user
-# 2) add the following configuration stanza to weewx.conf
+# 2) add the EmonCMS configuration stanza to weewx.conf
 # 3) restart weewx
 #
+# Minimal configuation to upload to emoncms.org.  This will use weewx variable
+# names as the feed names and default formatting.
+#
 # [[EmonCMS]]
 #     token = TOKEN
+#
+# Configuration with mapping of weewx names to custom feed names and local
+# emoncms server.  When an input_map is specified, only variables in that
+# map will be uploaded.
+#
+# [[EmonCMS]]
+#     station = ts
+#     server_url = http://192.168.0.1/emoncms/input/post.json
+#     [[[input_map]]]
+#         barometer = barometer_inHg:%.3f
+#         outTemp = outTemp_F:%.1f
+#         outHumidity = outHumidity:%03.0f
+#         windSpeed = windSpeed_mph:%.2f
+#         windDir = windDir:%03.0f
 
 import Queue
 import sys
@@ -106,24 +120,12 @@ class EmonCMS(weewx.restx.StdRESTbase):
 
 class EmonCMSThread(weewx.restx.RESTThread):
 
-    _SERVER_URL = 'http://emoncms.org/input/post'
-    _FORMATS = {'barometer'   : 'barometer_inHg:%.3f',
-                'outTemp'     : 'outTemp_F:%.1f',
-                'outHumidity' : 'outHumidity:%03.0f',
-                'inTemp'      : 'inTemp_F:%.1f',
-                'inHumidity'  : 'inHumidity:%03.0f',
-                'windSpeed'   : 'windSpeed_mph:%.2f',
-                'windDir'     : 'windDir:%03.0f',
-                'windGust'    : 'windGust_mph:%.2f',
-                'dewpoint'    : 'dewpoint_F:%.1f',
-                'rain24'      : 'rain24_in:%.2f',
-                'hourRain'    : 'hourRain_in:%.2f',
-                'dayRain'     : 'dayRain_in:%.2f',
-                'radiation'   : 'radiation:%.2f',
-                'UV'          : 'UV:%.2f'}
+    _DEFAULT_SERVER_URL = 'http://emoncms.org/input/post'
 
     def __init__(self, queue, token, database_dict,
-                 station=None, server_url=_SERVER_URL, skip_upload=False,
+                 station=None,
+                 server_url=_DEFAULT_SERVER_URL, input_map=None,
+                 skip_upload=False,
                  post_interval=None, max_backlog=sys.maxint, stale=None,
                  log_success=True, log_failure=True,
                  timeout=60, max_tries=3, retry_wait=5):
@@ -141,6 +143,7 @@ class EmonCMSThread(weewx.restx.RESTThread):
         self.token = token
         self.station = station
         self.server_url = server_url
+        self.input_map = input_map
         self.skip_upload = to_bool(skip_upload)
 
     def process_record(self, record, archive):
@@ -159,15 +162,25 @@ class EmonCMSThread(weewx.restx.RESTThread):
             raise weewx.restx.FailedPost("Server returned '%s'" % txt)
 
     def get_url(self, record):
+        # if there is a station name, use that as a prefix to every variable
         prefix = ''
         if self.station is not None:
             prefix = '%s_' % urllib.quote_plus(self.station)
+        # if there is an input map, use it to map the weewx variables into
+        # emoncms feed variables and formats.  otherwise, use the raw weewx
+        # names and default formatting.
         data = []
-        for k in self._FORMATS:
-            v = record.get(k)
-            if v is not None:
-                s = self._FORMATS[k] % v
+        if self.input_map is not None:
+            for k in self.input_map:
+                v = record.get(k)
+                if v is not None:
+                    s = self.input_map[k] % v
+                    data.append('%s%s' % (prefix, s))
+        else:
+            for k in record:
+                s = '%s:%s' % (k, record[k])
                 data.append('%s%s' % (prefix, s))
+        # assemble the complete url from all the variables and values
         url = '%s?apikey=%s&time=%s&json={%s}' % (
             self.server_url, self.token, record['dateTime'], ','.join(data))
         logdbg('url: %s' % url)
